%% Initiate
performOwnLoad = false;


files = {...
    '15.mat';...
    '20.mat';...
    '25.mat';...
    '30.mat';...
    '40.mat';...
    '50.mat';...
    };



putDir = 'averageFaces';
dateCompensation = [1 1 1 2 2];
verbose = true;
%% Load all files
if(~performOwnLoad)
    allPatients = cell(6,1);
    for i = 1:length(files)
        disp(['Loading group ' num2str(i)]);
        allPatients{i} = load(fullfile('C:\Users\robin\Desktop\Results',files{i})).patients;
    end
end

%% Filter scans
c = 1;
excludePatients = cell(1,6);
nPats = zeros(1,6);
nPatsHair = zeros(1,6);
nExcluded = zeros(1,6);
for i = 1 : length(files)
    p = allPatients{i};
    f = faulthyMeshes{i};
    np = 0; % Number of patients
    nh = 0; % Number of included with hair
    ne = 0; % Number of excluded patients
    exclude = zeros(1, length(f));
    for j = 1 : length(p)
        if(f(j) == 1)
            c = c + 1;
            np = np + 1;
            continue;
        end
        
        if(includeNoNead(c) == 1)
            disp(['We will now also include ' p(j).photo ])
            nh = nh + 1;
            exclude(j) = 1;
        else
            ne = ne + 1;
            exclude(j) = 2;
        end
        
        c = c + 1;
    end
    nPats(i) = np;
    nPatsHair(i) = nh;
    nExcluded(i) = ne;
    excludePatients{i} = exclude;
end

%% Prepare data filtering
% Determine the places of the forehead
idxNoHead = 1:3602;
idxHead = 1:3602;
deleteIdx = zeros(size(idxNoHead));
faces = face_template{3,1};
foreheadfaceidx = [face_template{2,6}; face_template{2,16}];
for i = 1:length(foreheadfaceidx)
    f = faces(foreheadfaceidx(i), :);
    deleteIdx(f(1)) = deleteIdx(f(1)) + 1;
    deleteIdx(f(2)) = deleteIdx(f(2)) + 1;
    deleteIdx(f(3)) = deleteIdx(f(3)) + 1;
end

deleteIdx(deleteIdx > 0) = 1;
idxHead = idxHead(logical(deleteIdx));
deleteIdx = ~deleteIdx;

idxNoHead = idxNoHead(deleteIdx);
idx = 1:3602;

%% Create average faces
perPatient = cell(1,6);

% Step 4 - Create average faces

averageFaces = cell(1,6);
for i = 1:length(files)
    pat = allPatients{i};
    averageFace = struct;
    averageFace.vertices = zeros(size(pat(1).finalAligned.vertices));
    averageFace.faces = pat(1).finalAligned.faces;
    
    factor = 1 / (nPats(i) + nPatsHair(i));
    factorOnlyHair = 1 / nPats(i);
    exclude = excludePatients{i};
    perPatientItem = cell(1,1);
    perPatientCounter = 1;
    for j = 1:length(pat)
        % All
        if(exclude(j) == 0)
            perPatientItem{perPatientCounter} = pat(j).finalAligned.vertices;
            perPatientCounter = perPatientCounter + 1;
            averageFace.vertices(idxNoHead,:) = averageFace.vertices(idxNoHead,:) + (factor .* pat(j).finalAligned.vertices(idxNoHead,:));
            averageFace.vertices(idxHead,:) = averageFace.vertices(idxHead,:) + (factorOnlyHair .* pat(j).finalAligned.vertices(idxHead,:));
            % Exclude head
        elseif(exclude(j) == 1)
            temp = pat(j).finalAligned.vertices;
            temp(idxHead,:) = -1000000;
            perPatientItem{perPatientCounter} = temp;
            perPatientCounter = perPatientCounter + 1;
            averageFace.vertices(idxNoHead,:) = averageFace.vertices(idxNoHead,:) + (factor .* pat(j).finalAligned.vertices(idxNoHead,:));
        else
            disp(['Removed' num2str(i) '-' num2str(j)]);
            
        end
    end
    perPatient{i} = perPatientItem;
    averageFaces{i} = averageFace;
end

%% Show the average faces
if(verbose)
    colors = jet(6);
    figure; hold on;
    for i = 1:length(averageFaces)
        stlwrite(fullfile(putDir, [num2str(i) '.stl']), averageFaces{i});
        RB_showFace(averageFaces{i}, colors(i,:));
        daspect([1 1 1]);
    end
end

%% Create distancemaps
distanceMaps = cell(1,length(averageFaces) - 1);
for i = 1:length(averageFaces) - 1
    moving = averageFaces{i + 1};
    target = averageFaces{i};
    
    % Get the distances
    distanceMaps{i} = DistanceMapCPD_Directional(moving.vertices, target.vertices, target.faces);
    
    % Compensate for time delta differences
    distanceMaps{i} =  distanceMaps{i} ./ dateCompensation(i);
    
    % Maximum 4 decimals
    distanceMaps{i} = round(distanceMaps{i} * 4)/4;
    
    if(verbose)
        
        f = figure;
        p = RB_showFace(moving);
        shading interp
        lighting flat;
        material dull
        daspect([1 1 1]);
        set(p, 'cdata', distanceMaps{i});
        colormap(othercolor('BuDRd_18'))
        caxis([-4 4]);
        h = colorbar
        set(h, 'ylim', [0 4])
        ylim([-100 100]);
        zlim([-100 65]);
        xlim([-70 70]);
        
        set(gca,'visible','off')
        set(gca,'color','none')
        exportgraphics(gcf,[num2str(i) '1.png'],'BackgroundColor','none','Resolution',600)
    end
end

%% Calculate distances per region
%%

% Eye
regions.eye = [para.face_template{5,2}; para.face_template{5,12}];

% Subnasal
regions.subnasal = [para.face_template{5,4}; para.face_template{5,14}];

% Chin
regions.chin = [para.face_template{5,3}; para.face_template{5,13}];

% Cheek
regions.cheek = [para.face_template{5,5}; para.face_template{5,15}];

% Nose
regions.nose = [para.face_template{5,9}; para.face_template{5,19}];

% Forehead
regions.forehead =[ para.face_template{5,6}; para.face_template{5,16}];

% Other
regions.other = [para.face_template{5,8}; para.face_template{5,18}];

% Mouth
regions.mouth = [...
    para.face_template{5,7}; para.face_template{5,17};...
    para.face_template{5,10}; para.face_template{5,20};...
    para.face_template{5,11}; para.face_template{5,21}...
];

%% Get data

t = cell(31,5);

t(1,1:5) = {'Region', 'Group','Mean growth (mm)','std (mm)','95% CI'};
ages = {'1.5-2.0','2.0-2.5','2.5-3.0','3.0-4.0','4.0-5.0'};

for i = 1:5
    % Get the coordinates
    moving = averageFaces{i}.vertices;
    target = averageFaces{i + 1}.vertices;
    
    % Calculate the distances
    distances = sqrt((moving(:,1) - target(:,1)).^2 + (moving(:,2) - target(:,2)).^2 + (moving(:,3) - target(:,3)).^2);
    
    if(i > 3)
       distances = distances ./ 2; 
    end
     
    % Total group
    total = distances;
    total([regions.mouth; regions.eye]) = [];
    t((i-1) + 2, 1:5) = {'', ages{i}, sprintf('%0.2f',mean(total)), sprintf('%0.2f',std(total)), ['[' sprintf('%0.2f',prctile(total, 5)) ' to ' sprintf('%0.2f',prctile(total, 95)) ']'] };
    
    nose = distances(regions.nose);
    t((i-1) + 7, 1:5) = {'', ages{i}, sprintf('%0.2f',mean(nose)), sprintf('%0.2f',std(nose)), ['[' sprintf('%0.2f',prctile(nose, 5)) ' to ' sprintf('%0.2f',prctile(nose, 95)) ']'] };
     
    forehead = distances(regions.forehead);
    t((i-1) + 12, 1:5) = {'', ages{i}, sprintf('%0.2f',mean(forehead)), sprintf('%0.2f',std(forehead)), ['[' sprintf('%0.2f',prctile(forehead, 5)) ' to ' sprintf('%0.2f',prctile(forehead, 95)) ']'] };
    
    subnasal = distances(regions.subnasal);
    t((i-1) + 17, 1:5) = {'', ages{i}, sprintf('%0.2f',mean(subnasal)), sprintf('%0.2f',std(subnasal)), ['[' sprintf('%0.2f',prctile(subnasal, 5)) ' to ' sprintf('%0.2f',prctile(subnasal, 95)) ']'] };
    
    chin = distances(regions.chin);
    t((i-1) + 22, 1:5) = {'', ages{i}, sprintf('%0.2f',mean(chin)), sprintf('%0.2f',std(chin)), ['[' sprintf('%0.2f',prctile(chin, 5)) ' to ' sprintf('%0.2f',prctile(chin, 95)) ']'] };
    
    cheek = distances(regions.cheek);
    t((i-1) + 27, 1:5) = {'', ages{i}, sprintf('%0.2f',mean(cheek)), sprintf('%0.2f',std(cheek)), ['[' sprintf('%0.2f',prctile(cheek, 5)) ' to ' sprintf('%0.2f',prctile(cheek, 95)) ']'] };
   
end
%%
writecell(t, 'results_regions.xlsx')






