%% Start Script Facial Analysis
clear all;
close all;
clc;

%% Parameters
startPath = '\';
templatePath = '\template_1.mat';
parameterPath = '\parameters.mat';
path2meshmonk = '\meshmonk';
path2FAT = '\FacialAssessmentToolbox';

addpath(genpath(path2meshmonk));
addpath(genpath(path2FAT));

disp(startPath);
%% Load template and parameters
template = load(templatePath);
face_template = template.face_template;
parameters = load(parameterPath);
para = parameters.para;
para.face_template = face_template;
para = findBackVerts(para);
face_template = para.face_template;

%% Load data
patients = findOBJ2_withUI_4(startPath,struct(), false);
%patients(2:end) =[];
patients = readAllOBJ_v3(patients,0,1);

%% Align
patients = RB_AlignMesh(patients);

%% Get starting points to start template matching
for i = 1:length(patients)
    coords = patients(i).coordsAligned;
    patients(i).mesh.Auto5p_landmarks = zeros(1,5);
    patients(i).mesh.Auto5p_landmarks(1) = dsearchn(patients(i).mesh.vertices,coords(11,:));
    patients(i).mesh.Auto5p_landmarks(2) = dsearchn(patients(i).mesh.vertices,coords(12,:));
    patients(i).mesh.Auto5p_landmarks(3) = dsearchn(patients(i).mesh.vertices,coords(10,:));
    patients(i).mesh.Auto5p_landmarks(4) = dsearchn(patients(i).mesh.vertices,coords(14,:));
    patients(i).mesh.Auto5p_landmarks(5) = dsearchn(patients(i).mesh.vertices,coords(13,:));
end

%% Pre-Align the template on the faces
templateVerts = face_template{2,1};
templateFaces = face_template{3,1};
templateCoords = face_template{4,1};

mainTemplate = struct;
mainTemplate.v = face_template{2,1};
mainTemplate.f = face_template{3,1};

Template = shape3D;
Template.Faces = templateFaces;
Template.Vertices = templateVerts;
templateLandmarks = Template.Vertices(templateCoords,:);

numPats = length(patients);
%parfor i = 1:numPats
    parfor i = 1:numPats;
    currentPatient = patients(i);
    
    RM= ShapeMapper;
    RM.NumIterations = 30;
    RM.TransformationType = 'rigid';
    RM.UseScaling = true;
    
    % settings for determining the correspondences
    RM.CorrespondencesNumNeighbours = 3; % number of neighbours to use to estimate correspondences
    RM.CorrespondencesFlagThreshold = 0.9;
    RM.CorrespondencesSymmetric = true; % if true correspondences are estimated from the template to target and target to template and combined -
    RM.CorrespondencesEqualizePushPull = false;
    RM.InlierKappa = 3;
    RM.InlierUseOrientation = true; % use surface normal direction to determine of point is inlier/outlier
    RM.FlagFloatingBoundary = true; % ignore points that correspondences to the edge of Floating surface - only applicable if using SymmetricCorrespndences
    RM.FlagTargetBoundary = true;% ignore points that correspondences to the edge of Target surface
    RM.FlagTargetBadlySizedTriangles = true; % ignore points that match to regions of abnormally sized triangles
    RM.TriangleSizeZscore = 6; % threshold to determine which triangles are abnormally sized
    RM.UpSampleTarget = false; % will upsample the target mesh. if meshes are irregular it can help to set this to true
    
    % Set up non rigid ICP with ShapeMapper with 'nonrigid' transformation type
    NRM = ShapeMapper;
    NRM.TransformationType = 'nonrigid';
    NRM.NumIterations = 200;
    NRM.CorrespondencesSymmetric = true;
    NRM.CorrespondencesNumNeighbours = 3;
    NRM.CorrespondencesFlagThreshold = 0.9;
    NRM.CorrespondencesUseOrientation = true;
    NRM.CorrespondencesEqualizePushPull =false;
    NRM.InlierUseOrientation = true;
    NRM.FlagFloatingBoundary = true;
    NRM.FlagTargetBoundary = true;
    NRM.FlagTargetBadlySizedTriangles = true;
    NRM.TriangleSizeZscore = 6;
    NRM.UpSampleTarget = true;
    NRM.TransformSigma = 3;
    NRM.TransformNumViscousIterationsStart = 200;
    NRM.TransformNumViscousIterationsEnd = 1;
    NRM.TransformNumElasticIterationsStart = 200;
    NRM.TransformNumElasticIterationsEnd = 1;
    NRM.TransformNumNeighbors = 80;
    
    [path, name, ext] = fileparts(patients(i).csvPath);
    Patient= shape3D;
    Patient.Vertices = patients(i).mesh.vertices;
    Patient.Faces = patients(i).mesh.faces;
    
    patientLandmarks = zeros(5,3);
    patientLandmarks(1,:) = patients(i).coordsAligned(3,1:3);
    patientLandmarks(2,:) = patients(i).coordsAligned(4,1:3);
    patientLandmarks(3,:) = patients(i).coordsAligned(10,1:3);
    patientLandmarks(4,:) = patients(i).coordsAligned(13,1:3);
    patientLandmarks(5,:) = patients(i).coordsAligned(14,1:3);
    
    meshmonk = struct;
    
    meshmonk.landmarks = patientLandmarks;
    
    disp(['Processing patient ' num2str(i) ' of ' num2str(numPats)]);
    
    % Calculate the transform between the landmarks of the template and
    % patient (?Procrustes?)
    
    T = computeTransform(templateLandmarks,meshmonk.landmarks,true);
    meshmonk.templateT = T;
    
    % Apply the T to the template
    forTemplate = applyTransform(Template,T);
    
    % Check normals
    if ~normalsConsistent(Patient, forTemplate)
        Patient.FlipNormals = true;
    end
    
    % Refine
    RM.FloatingShape = clone(forTemplate);
    RM.TargetShape = Patient;
    RM.map();
    
    NRM.Display = false; % set to false if you don't want to see the mapping
    NRM.TargetShape = Patient;
    NRM.FloatingShape = clone(RM.FloatingShape);
    NRM.map();
    
    mappedPatient = NRM.FloatingShape;
    patients(i).meshmonk.template = meshmonk;
    patients(i).meshmonk.vertices = mappedPatient.Vertices;
    patients(i).meshmonk.faces = mappedPatient.Faces;
end

%% Align the faces on the reference frame

    template = load('template_1.mat');
face_template = template.face_template;
parameters = load('parameters.mat');
para = parameters.para;
para.face_template = face_template;
para = findBackVerts(para);
face_template = para.face_template;


    mainTemplate.v = face_template{2,1};
mainTemplate.f = face_template{3,1};
templateCoords = face_template{4,1};
templateC = mainTemplate.v(templateCoords,:);
for j = 1:length(patients)
    tic
        coords = patients(j).mesh.Auto5p_landmarks';
        [~, ~, T] = procrustes(patients(j).mesh.vertices(coords,:), templateC);
        vScaled = T.b .* mainTemplate.v;
        pcMoving = pointCloud(patients(j).meshmonk.vertices);
        pcTarget = pointCloud(vScaled);
        [tform, pcFinal] = pcregistericp(pcMoving, pcTarget, 'MaxIterations', 100, 'Tolerance', [0.001, 0.005]);
        [mAligned, patients(j).coordsAligned2] = RB_RotateObject(patients(j).meshmonk, patients(j).coordsAligned, invert(tform).Rotation, [0 0 0]);
        prp = mean(patients(j).coordsAligned2(3:4,:));
        [mAligned, patients(j).coordsAligned2] = RB_TranslateObject(mAligned, patients(j).coordsAligned2, -[prp(1) mean(patients(j).coordsAligned2(8:9,2)) prp(3)]);
        
        vAligned = patients(j).meshmonk.vertices * invert(tform).Rotation;
        vAligned = vAligned - tform.Translation;
        mean(mAligned.vertices)
        patients(j).finalAligned = mAligned;
        toc  
end



disp(startPath);
